#!/bin/bash
#
# Configures the per-service log files for a running composed service.
# This script does not yet handle the aggregate logging functionality.
#
# Author: Kovács Zoltán <kovacsz@marcusconsulting.hu>
# License: GNU/GPL v3+ (https://www.gnu.org/licenses/gpl-3.0.en.html)
#
# 2025-03-29 v0.3
# mod: no longer repeats existing log lines.
# 2025-02-26 v0.2
# fix: a silly typo (commandtring) blocked the startup.
# 2025-02-02 v0.1 Initial release

# Accepted environment variables and their defaults.
PAR_BASEDIR=${PAR_BASEDIR:-""}			# Service's base folder
PAR_AGGLOGS=${PAR_AGGLOGS:-""}			# Not empty for aggregated logs

# Other initialisations.
LOGDIR="logs"
LOGFILE="logs/docker-compose.log"		# For aggregated log
YMLFILE="docker-compose.yml"

# Messages.
MSG_DOESNOTRUN="This service doesn't run."
MSG_MISSINGDEP="Fatal: missing dependency"
MSG_MISSINGYML="Fatal: didn't find the docker-compose.yml file"

# Checks the dependencies.
TR=$(which tr 2>/dev/null)
if [ -z "$TR" ]; then echo "$MSG_MISSINGDEP tr."; exit 1 ; fi
for item in cut date dirname docker grep ps readlink tail
do
    if [ -n "$(which $item)" ]
    then export $(echo $item | "$TR" '[:lower:]' '[:upper:]' | "$TR" '-' '_')=$(which $item)
    else echo "$MSG_MISSINGDEP $item." >&2; exit 1; fi
done
# All dependencies are available via "$THECOMMAND" (upper case) call.
#
# Let's find which version of docker-compose is installed.
if [ $($DOCKER compose version >/dev/null 2>&1; echo $?) -eq 0 ]; then
    # We'll use v2 if it is available.
    DOCKER_COMPOSE="$DOCKER"
    commandstring="compose"
else
    # Otherwise falling back to v1.
    DOCKER_COMPOSE="$(which docker-compose)"
    commandstring=""
fi
# One of the two is mandatory.
if [ -z "$DOCKER_COMPOSE" ];then echo "$MSG_MISSINGDEP docker-compose" >&2; exit 1; fi
# Below docker-compose should be called as "$DOCKER_COMPOSE" $commandstring sequence.

# Where I'm?
# https://gist.github.com/TheMengzor/968e5ea87e99d9c41782
SOURCE="$0"
while [ -h "$SOURCE" ]; do
    # resolve $SOURCE until the file is no longer a symlink
    SCRPATH="$( cd -P "$("$DIRNAME" "$SOURCE" )" && pwd )" #"
    SOURCE="$("$READLINK" "$SOURCE")"
    # if $SOURCE was a relative symlink, we need to resolve it
    # relative to the path where the symlink file was located
    [[ $SOURCE != /* ]] && SOURCE="$SCRPATH/$SOURCE"
done; SCRPATH="$( cd -P "$("$DIRNAME" "$SOURCE" )" && pwd )" #"

# Searches the base folder, containing a docker-compose.yml file.
# Called from the base folder (./)?
BASE_DIR="$PAR_BASEDIR"
TEST_DIR="$SCRPATH"
[[ -z "$BASE_DIR" ]] && [[ -r "$TEST_DIR/$YMLFILE" ]] && BASE_DIR="$TEST_DIR"
# Called from ./tools?
TEST_DIR="$("$DIRNAME" "$TEST_DIR")"
[[ -z "$BASE_DIR" ]] && [[ -r "$TEST_DIR/$YMLFILE" ]] && BASE_DIR="$TEST_DIR"
# Called from ./tools/*.d?
TEST_DIR="$("$DIRNAME" "$TEST_DIR")"
[[ -z "$BASE_DIR" ]] && [[ -r "$TEST_DIR/$YMLFILE" ]] && BASE_DIR="$TEST_DIR"
# On failure gives it up here.
if [ -z "$BASE_DIR" -o ! -r "$BASE_DIR/$YMLFILE" ]; then
    echo "$MSG_MISSINGYML" >&2; exit 1
fi

# Quits silently if the service is not running.
[[ -z "$(cd "$BASE_DIR"; "$DOCKER_COMPOSE" $commandstring ps --services --filter "status=running")" ]] &&  exit

# Checks the logger(s), starts if it isn't running.
# These process(es) will automatically terminate when the composed service stops.
if [ -n "$PAR_AGGLOGS" ]; then
    # Aggregated logs
    # (cd "$BASE_DIR"; "$DOCKER_COMPOSE" $commandstring logs --no-color -t -f >> "$BASE_DIR/$LOGFILE" &)
    :
else
    # Separate logs, each for every running service.
    for service in $(cd "$BASE_DIR"; "$DOCKER_COMPOSE" $commandstring ps --services) ""
    do
        if [ -n "$service" ]; then
	    # Converts the service's name to an actual running container's name.
	    container="$("$DOCKER" inspect -f '{{.Name}}' $(cd "$BASE_DIR"; "$DOCKER_COMPOSE" $commandstring ps -q "$service") | "$CUT" -c2-)"
	    # It will not start the log process if it already exists.
	    if [ -z "$("$PS" auxww | "$GREP" "$DOCKER" | "$GREP" 'logs' | "$GREP" "$container" )" ]; then
		# Determines the last existing log line in the log file (if any).
		logline="$("$TAIL" -n1 "$BASE_DIR/$LOGDIR/$service.log" 2>/dev/null)"
		# Gets the timestamp from this line.
		[[ -n "$logline" ]] \
		    && timestamp="$(echo "$logline" | "$CUT" -d' ' -f1 2>/dev/null)" \
		    || timestamp="invalid"
		# Checks the validity.
		[[ $("$DATE" -d "$timestamp" >/dev/null 2>&1; echo $?) -eq 0 ]] \
		    && since="$timestamp" \
		    || since="1970-01-01T00:00Z"
		# Only logs the new lines (actually repeats the last one - TODO!).
        	"$DOCKER" logs -t --since "$since" -f $container >> "$BASE_DIR/$LOGDIR/$service.log" 2>&1 &
	    fi
        fi
    done
fi

# That's all, Folks!
